/********************************************************
Workspace: IAR_ARM+SPL
MCU      : STM32f050f4/6 (||+F031)
Project  : SW_REG
Version  :
Date     : MAY 2020
*********************************************************/

#include <stm32f0xx.h>
#include <stdbool.h>
#include <uchar.h>
#include <math.h>

#include "stm32f0xx_gpio.c"
#include "stm32f0xx_dma.c"

#include "bitset.c"
#include "defines.c"
#include "periph_cfg.c"
#include "beep.c"
#include "nvic.c"
#include "p_pwm.c"
#include "led.c"
#include "kb.c"
#include "btn_processing.c"

#include "interrupts.c"

void devcon(void);

void main()
{
 cfg_rcc();
 cfg_gpio();
 cfg_t1();
 cfg_dma_t1();
 cfg_systic();
 cfg_nvic();

 calculate_bsw();
 set_new_frq(30);

 __enable_interrupt();

 led_pins_init();
 led(green, false);
 led(red,   false);

 btn_init();

 ENDLESS
 {
  static bool on_off = true;
  static bool on_off_tmp;
  static int tmp_ccr;

  //  /
  //           
  if(on_off_tmp != on_off)
  {
   //    
   if(on_off == false)
   {
    static bool trg;
    int k = TIM1->CCR2;

    k = (int)TIM1->ARR/2 - k;

    /*
             r   arr/2
        arr/2 -ccr  
    */
    if(trg == false)
    {
     if(k >= 0) trg = true;
    }
    //     arr/2
    else
    {
     // r   arr/2
     if(k < 0)
     {
      TIM1->BDTR = 0;
      trg = false;

      on_off_tmp = on_off;
     }
    }
   }
   //    
   else
   {
    static bool trg;
    int k = TIM1->CCR2;

    k = k - tmp_ccr;
    /*
             r   
          
    */
    if(trg == false)
    {
     // ccr 
     if(k >= 0) trg = true;
    }
    //     ccr
    else
    {
     // r 
     if(k < 0)
     {
      TIM1->BDTR = TIM_BDTR_MOE;
      trg = false;

      on_off_tmp = on_off;
     }
    }
   }
  }
  tmp_ccr = TIM1->CCR2;

  //   50   
  if(!SB_TIM)
  {
   static bool k_change;       //   
   static int f_val = 2;       //  

   SB_TIM = TSEC/50;

   btn_scan();
   if(key_processing(&f_val, &on_off, btn)==true) k_change = true;

   led(green, true^on_off);
   led(red,   on_off);

   static int t_cnt = 0;
   //     
   if(t_cnt<50) t_cnt++;
   else
   {
    t_cnt = 0;

    //  
    if(k_change == true)
    {
     int rcr;

     k_change = false;

     switch(f_val)
     {
      case 0: rcr = 4;  break; //50Hz
      case 1: rcr = 5;  break; //40Hz
      case 2: rcr = 6;  break; //33Hz
      case 3: rcr = 7;  break; //28Hz
      case 4: rcr = 9;  break; //22Hz
      case 5: rcr = 15; break; //13Hz
      default: rcr = 4; break;
     }

     set_new_frq_by_rcr(rcr);
    }
   }
  }
 }
}

