#define PWM_TOP  0xff
#define SPWM_FRQ (SYSTEM_FRQ/(PWM_TOP+1)/2) //15625
#define WFRQ_MIN 10
#define WFRQ_MAX 50
#define MAX_AMP  0x100
#define MIN_RCR  4

#define CCR_DT   5

//         RCR = 4
//    10     RCR   4..24
#define SW_MSIZE (SPWM_FRQ/WFRQ_MAX/MIN_RCR) //78

//   
s16 B_wave[SW_MSIZE];

//    
//[    DMA,   ]
//[       ]
//[     CCR ]
s16 Hf_wave[2][SW_MSIZE][2];
u32 Bank_dma = 0;

u32 RCR_new = 4;

//    
void calculate_bsw(void)
{
 int i;
 const float y = PI*2/SW_MSIZE;
 const int ss  = PWM_TOP/2;

 for(i=0;i<SW_MSIZE/2;i++) B_wave[i] = s16(sin(y*i)*ss);

 const int tmp = i;

 for(;i<SW_MSIZE;i++) B_wave[i] = -B_wave[i-tmp];
}

//         
//           dma
void set_wamp(u32 amp, u32 const bank=0)
{
 if(amp > MAX_AMP*2) amp = MAX_AMP*2;

 for(int i=0;i<SW_MSIZE;i++)
 {
  s32 tmp = (s32)B_wave[i]*amp >>8;
  Hf_wave[bank][i][0] = Hf_wave[bank][i][1] = PWM_TOP/2 + tmp;

  //   deadtime  CCR
  Hf_wave[bank][i][0] += CCR_DT; //CCR2, PWM2
  Hf_wave[bank][i][1] -= CCR_DT; //CCR3, PWM1

  //    
  //   
  if(Hf_wave[bank][i][0] > PWM_TOP)
  {
   Hf_wave[bank][i][1] -= Hf_wave[bank][i][0] - PWM_TOP;
   Hf_wave[bank][i][0]  = PWM_TOP;
  }
  //   
  else if(Hf_wave[bank][i][1] < 0)
  {
   Hf_wave[bank][i][0] -= Hf_wave[bank][i][1];
   Hf_wave[bank][i][1]  = 0;
  }
 }
}

//   set_wamp(..)
void change_bank(u32 bank)
{
 //     
 if(bank>1)bank = 1;

 Bank_dma = bank;

 //   IFCR
 DMA1->IFCR |= DMA_IFCR_CGIF5;
 __DSB();

 //      dma
 DMA1_Channel5->CCR |= DMA_CCR_TCIE;
}

//  RCR     
int get_rcr(int frq)
{
 if(frq > WFRQ_MAX)      frq = WFRQ_MAX;
 else if(frq < WFRQ_MIN) frq = WFRQ_MIN;

 return (SPWM_FRQ*1000/SW_MSIZE/frq)/1000;
}

//        
int get_amp(const int rcr_val)
{
 #define KUC (220*220/(50*50)) //19

 //      RCR
 const int swf = SPWM_FRQ/SW_MSIZE/rcr_val;
 //  
 const int uvx = (int)sqrt(float(KUC * swf*swf));

 return MAX_AMP * uvx /110;
}

void set_new_frq(int frq)
{
 int tmp;
 int bank;

 RCR_new = get_rcr(frq);

 tmp = get_amp(RCR_new);

 bank = Bank_dma;
 bank ^= 1;
 set_wamp(tmp, bank);
 change_bank(bank);
}

void set_new_frq_by_rcr(int rcr)
{
 int amp;
 int bank;
 static const int min_val = get_rcr(WFRQ_MAX);
 static const int max_val = get_rcr(WFRQ_MIN);

 if(rcr < min_val)      rcr = min_val;
 else if(rcr > max_val) rcr = max_val;

 RCR_new = rcr;

 amp = get_amp(RCR_new);

 bank = Bank_dma;
 bank ^= 1;
 set_wamp(amp, bank);
 change_bank(bank);
}




